/**
 * Lex/flex specification for C implementation of JM parser.
 * J. Fenwick, May 2006.
 * Minor edits by R. Topor.
 */

%{
#include "ass1.tab.h"
#define YY_DECL int getToken()
int curchar;
%}

OPENCOMMENT 	\/\/
DIGIT 		[0-9]
LETTER 		[_a-zA-Z]
QUOTE 		\"
QUOTESTR 	\\\"
INQUOTECHAR 	([^\n\"\\]|{QUOTESTR}|\\\\)
INTVAL 		{DIGIT}+
DVAL 		{DIGIT}+((\.{DIGIT}+(e[\+|-]{DIGIT}+)?)|(e[\+|-]{DIGIT}+ ))
EXPONENT 	e[+-]{DIGIT}+
DOUBLEVAL 	{DIGIT}+((\.{DIGIT}+{EXPONENT}?)|{EXPONENT})
ID 		{LETTER}({LETTER}|{DIGIT})*
NEWLINE		\n

CLASS 	class
IF 	if
ELSE 	else
ELSEIF 	elseif
EXTENDS	extends
NEW 	new
NULL	null
RETURN 	return
THIS	this
VOID	void
WHILE 	while

LBRACK 	\[
RBRACK 	\]
LBRACE 	\{
RBRACE 	\}
MOD 	\%
PLUS 	\+
MINUS 	\-
TIMES 	\*
DIVIDES \/
LPAREN 	\(
RPAREN 	\)
EQ 	==
ASSIGN 	=
NEQ 	!=
LT 	\<
LEQ 	\<=
GT 	\>
GEQ 	\>=
SEMI 	;
COMMA 	,
DOT 	\.

WSPACE [ \t\n]

%option noyywrap

%%

{NEWLINE}	%{
		return NEWLINE; 
		%}
{OPENCOMMENT}	%{
		curchar=input();
		while (curchar!=EOF && curchar!='\n') {curchar=input();}
	    	%}
({QUOTE}{QUOTE})|({QUOTE}{INQUOTECHAR}*{QUOTE})	%{
		return STRING;
		%}
{QUOTE}{INQUOTECHAR}*	%{
		return ERROR;
		%}

{WSPACE}	{ /*protect whitespace from what follows*/ }

{INTVAL}	%{ 
		return INTEGER; 
		%}
{DOUBLEVAL}	%{ 
		return DOUBLE; 
		%}

{CLASS}		%{ 
		return CLASS; 
		%}
{EXTENDS}	%{ 
		return EXTENDS; 
		%}
{ELSE}		%{ 
		return ELSE; 
		%}
{ELSEIF}	%{ 
		return ELSEIF; 
		%}
{IF}		%{ 
		return IF; 
		%}
{NEW}		%{ 
		return NEW; 
		%}
{RETURN}	%{ 
		return RETURN; 
		%}
{THIS}		%{ 
		return THIS; 
		%}
{VOID}		%{ 
		return VOID; 
		%}
{WHILE}		%{ 
		return WHILE; 
		%}
(true)|(false)	%{ 
		return BOOLEAN; 
		%}
{ID}		%{ 
		return ID; 
		%}

{LBRACK}	return LBRACK;
{RBRACK}	return RBRACK;
{LBRACE}	return LBRACE;
{RBRACE}	return RBRACE;
{MOD}		return MOD;
{PLUS}		return PLUS;
{MINUS}		return MINUS;
{TIMES}		return TIMES;
{DIVIDES}	return DIVIDES;
{LPAREN}	return LPAREN;
{RPAREN}	return RPAREN;
{EQ}		return EQ;
{ASSIGN}	return ASSIGN;
{NEQ}		return NEQ;
{LT}		return LT;
{LEQ}		return LEQ;
{GT}		return GT;
{GEQ}		return GEQ;
{SEMI}		return SEMI;
{COMMA}		return COMMA;
{DOT}		return DOT;
.		%{ 
		return ERROR; 
		%}

%%

